import React from 'react';
import { BrowserRouter as Router, Route, Switch } from "react-router-dom";

import Navigation from './Navigation';
import Footer from './Footer';
import Home from './Home';
import About from './About';

// Maps
import Pigeon from './Pigeon';
import Mapbox from './Mapbox';
import Leaflet from './Leaflet';
import Google from './Google';

export class App extends React.Component{
    constructor(props) {
        super(props)

        this.state = {
            mapConfig: {
                zoom: 15,
                path: [],
                coords: [48.336950, 14.320570],
                width: 600,
                height: 400,
                defaultZoom: 15,
                distance: 0,
            }
        }
    }

    updateMapState = (key, value) => {
        let mapConfigCopy = JSON.parse(JSON.stringify(this.state.mapConfig));
        mapConfigCopy[key] = value;
        this.setState({mapConfig: mapConfigCopy})
    }

    calcDistance = () => {
        let dst = 0;
        let pathCopy = this.state.mapConfig.path.slice();
        for(let i=0; i<pathCopy.length-1; i++) {
            var radlat1 = Math.PI * pathCopy[i][0]/180;
            var radlat2 = Math.PI * pathCopy[i+1][0]/180;
            var theta = pathCopy[i][1]-pathCopy[i+1][1];
            var radtheta = Math.PI * theta/180;
            var d = Math.sin(radlat1) * Math.sin(radlat2) + Math.cos(radlat1) * Math.cos(radlat2) * Math.cos(radtheta);
            if (d > 1) {
                d = 1;
            }
            dst += Math.acos(d) * 180/Math.PI * 60 * 1.1515 * 1.609344;
        }
        this.updateMapState("distance", dst)
    }

    render(){
        return (
          <div>
              <Router>
                <Navigation />
                <Switch>
                  <Route path="/" exact component={Home} />
                  <Route path="/about" exact component={About} />
                  <Route exact path="/pigeon-map" render={() =>
                    <Pigeon
                        mapConfig={this.state.mapConfig}
                        updateMapState={this.updateMapState}
                        calcDistance={this.calcDistance}
                    />}
                  />
                  <Route path="/mapbox-gl" exact component={Mapbox} />
                  <Route path="/leaflet" exact component={Leaflet} />
                  <Route path="/google-map" exact component={Google} />
                </Switch>
                <Footer />
              </Router>
          </div>
        )
    }

}